//------------------------------------------------------------------------------
// Lamp : Open source game middleware
// Copyright (C) 2004  Junpei Ohtani ( Email : junpee@users.sourceforge.jp )
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

/** @file
 * 三次元テクスチャ座標ヘッダ
 * @author Junpee
 */

#ifndef TEX_COORD_3_H_
#define TEX_COORD_3_H_

#include <Core/System/Math.h>
#include <Core/Primitive/String.h>

namespace Lamp{

//------------------------------------------------------------------------------
/**
 * 三次元テクスチャ座標
 *
 * このクラスは継承しないで下さい。
 */
class TexCoord3{
public:
	//--------------------------------------------------------------------------
	// メンバ変数
	//--------------------------------------------------------------------------
	/// メンバ変数
	union{
		/// 各要素
		struct{
			/// U値
			float u;
			/// V値
			float v;
			/// W値
			float w;
		};

		/// 配列
		float array[3];
	};

	//--------------------------------------------------------------------------
	// 定数
	//--------------------------------------------------------------------------
	/// ゼロ座標
	static const TexCoord3 zero;

	/// 単位座標
	static const TexCoord3 unit;

	/// U単位座標
	static const TexCoord3 unitU;

	/// V単位座標
	static const TexCoord3 unitV;

	/// W単位座標
	static const TexCoord3 unitW;

	//--------------------------------------------------------------------------
	// コンストラクタ
	//--------------------------------------------------------------------------
	/**
	 * コンストラクタ
	 *
	 * このコンストラクタは初期値の設定を行わないため値は不定です。
	 */
	inline TexCoord3(){}

	/**
	 * コンストラクタ
	 * @param sourceU Uの初期値
	 * @param sourceV Vの初期値
	 * @param sourceW Wの初期値
	 */
	inline TexCoord3(float sourceU, float sourceV, float sourceW) :
		u(sourceU), v(sourceV), w(sourceW){
	}

	/**
	 * コンストラクタ
	 * @param source 初期値配列
	 */
	inline explicit TexCoord3(const float* const source) :
		u(source[0]), v(source[1]), w(source[2]){
	}

	//--------------------------------------------------------------------------
	// 値の設定
	//--------------------------------------------------------------------------
	/**
	 * 値の設定
	 * @param sourceU Uの設定値
	 * @param sourceV Vの設定値
	 * @param sourceW Wの設定値
	 */
	inline void set(float sourceU, float sourceV, float sourceW){
		u = sourceU;
		v = sourceV;
		w = sourceW;
	}

	/**
	 * 値の設定
	 * @param source 設定値配列
	 */
	inline void set(const float* const source){
		u = source[0];
		v = source[1];
		w = source[2];
	}

	//--------------------------------------------------------------------------
	// 演算
	//--------------------------------------------------------------------------
	/**
	 * 加算
	 * @param addCoord 加算する三次元テクスチャ座標
	 * @return 加算された三次元テクスチャ座標
	 */
	inline TexCoord3 operator +(const TexCoord3& addCoord) const{
		return TexCoord3(u + addCoord.u, v + addCoord.v, w + addCoord.w);
	}

	/**
	 * 減算
	 * @param subCoord 減算する三次元テクスチャ座標
	 * @return 減算された三次元テクスチャ座標
	 */
	inline TexCoord3 operator -(const TexCoord3& subCoord) const{
		return TexCoord3(u - subCoord.u, v - subCoord.v, w - subCoord.w);
	}

	/**
	 * 乗算
	 * @param mulValue 乗算する値
	 * @return 乗算された三次元テクスチャ座標
	 */
	inline TexCoord3 operator *(float mulValue) const{
		return TexCoord3(u * mulValue, v * mulValue, w * mulValue);
	}

	/**
	 * 乗算
	 * @param mulValue 乗算する値
	 * @param mulCoord 乗算する三次元テクスチャ座標
	 * @return 乗算された三次元テクスチャ座標
	 */
	inline friend TexCoord3 operator *(
		float mulValue, const TexCoord3& mulCoord){
		return TexCoord3(mulCoord.u * mulValue, mulCoord.v * mulValue,
			mulCoord.w * mulValue);
	}

	/**
	 * +演算子
	 * @return 三次元テクスチャ座標のコピー
	 */
	inline TexCoord3 operator +() const{ return *this; }

	/**
	 * -演算子
	 * @return 値の符号が反転した三次元テクスチャ座標
	 */
	inline TexCoord3 operator -() const{ return TexCoord3(-u, -v, -w); }

	//--------------------------------------------------------------------------
	// 代入演算
	//--------------------------------------------------------------------------
	/**
	 * 代入加算
	 * @param addCoord 加算する三次元テクスチャ座標
	 * @return 加算された三次元テクスチャ座標
	 */
	inline TexCoord3& operator +=(const TexCoord3& addCoord){
		u += addCoord.u;
		v += addCoord.v;
		w += addCoord.w;
		return *this;
	}

	/**
	 * 代入減算
	 * @param subCoord 減算する三次元テクスチャ座標
	 * @return 減算された三次元テクスチャ座標
	 */
	inline TexCoord3& operator -=(const TexCoord3& subCoord){
		u -= subCoord.u;
		v -= subCoord.v;
		w -= subCoord.w;
		return *this;
	}

	/**
	 * 代入乗算
	 * @param mulValue 乗算する値
	 * @return 乗算された三次元テクスチャ座標
	 */
	inline TexCoord3& operator *=(float mulValue){
		u *= mulValue;
		v *= mulValue;
		w *= mulValue;
		return *this;
	}

	//--------------------------------------------------------------------------
	// 論理演算
	//--------------------------------------------------------------------------
	/**
	 * 三次元テクスチャ座標が同じかどうか
	 * @param target 比較する三次元テクスチャ座標
	 * @return 同じ値であればtrueを返す
	 */
	inline bool operator ==(const TexCoord3& target) const{
		return ((u == target.u) && (v == target.v) && (w == target.w));
	}

	/**
	 * 三次元テクスチャ座標が同じかどうか
	 * @param target 比較する三次元テクスチャ座標
	 * @param epsilon 誤差
	 * @return 誤差の範囲内で同じ値であればtrueを返す
	 */
	inline bool epsilonEquals(
		const TexCoord3& target, float epsilon) const{
		Assert(epsilon >= 0.f);
		return (
			(Math::abs(u - target.u) <= epsilon) &&
			(Math::abs(v - target.v) <= epsilon) &&
			(Math::abs(w - target.w) <= epsilon));
	}

	/**
	 * 三次元テクスチャ座標が同じでないかどうか
	 * @param target 比較する三次元テクスチャ座標
	 * @return 同じでない値であればtrueを返す
	 */
	inline bool operator !=(const TexCoord3& target) const{
		return ((u != target.u) || (v != target.v) || (w != target.w));
	}

	/**
	 * 三次元テクスチャ座標が同じでないかどうか
	 * @param target 比較する三次元テクスチャ座標
	 * @param epsilon 誤差
	 * @return 誤差の範囲内で同じでない値であればtrueを返す
	 */
	inline bool notEpsilonEquals(
		const TexCoord3& target, float epsilon) const{
		Assert(epsilon >= 0.f);
		return (
			(Math::abs(u - target.u) > epsilon) ||
			(Math::abs(v - target.v) > epsilon) ||
			(Math::abs(w - target.w) > epsilon));
	}

	//--------------------------------------------------------------------------
	// その他
	//--------------------------------------------------------------------------
	/**
	 * 文字列化
	 * @return 三次元テクスチャ座標の文字列表記
	 */
	inline String toString() const{
		String returnString;
		returnString.format("( %.8f, %.8f, %.8f )", u, v, w);
		return returnString;
	}

	//--------------------------------------------------------------------------
private:

};

//------------------------------------------------------------------------------
} // End of namespace Lamp
#endif // End of TEX_COORD_3_H_
//------------------------------------------------------------------------------
